@@ -6,6 +6,7 @@ from __future__ import print_function
 from .core.alignment import align, broadcast, broadcast_arrays
 from .core.common import full_like, zeros_like, ones_like
 from .core.combine import concat, auto_combine
+from .core.computation import apply_ufunc, where, dot
 from .core.extensions import (register_dataarray_accessor,
                               register_dataset_accessor)
 from .core.variable import as_variable, Variable, IndexVariable, Coordinate
@@ -17,8 +18,11 @@ from .core.options import set_options
 from .backends.api import (open_dataset, open_dataarray, open_mfdataset,
                            save_mfdataset)
 from .backends.rasterio_ import open_rasterio
+from .backends.zarr import open_zarr
 
-from .conventions import decode_cf
+from .conventions import decode_cf, SerializationWarning
+
+from .coding.cftimeindex import CFTimeIndex
 
 try:
     from .version import version as __version__
@@ -27,6 +31,7 @@ except ImportError:  # pragma: no cover
                       'the source directory, please instead create a new '
                       'virtual environment (using conda or virtualenv) and '
                       'then install it in-place by running: pip install -e .')
+from .util.print_versions import show_versions
 
 from . import tutorial
 from . import ufuncs
