@@ -1,22 +1,21 @@
 """
 Functions for applying functions that act on arrays to xarray's labeled data.
-
-NOT PUBLIC API.
 """
+from __future__ import absolute_import, division, print_function
+from distutils.version import LooseVersion
 import functools
 import itertools
 import operator
+from collections import Counter
 
 import numpy as np
 
-from . import duck_array_ops
-from . import utils
+from . import duck_array_ops, utils, dtypes
 from .alignment import deep_align
 from .merge import expand_and_merge_variables
-from .pycompat import OrderedDict, dask_array_type
+from .pycompat import OrderedDict, dask_array_type, basestring
 from .utils import is_dict_like
 
-
 _DEFAULT_FROZEN_SET = frozenset()
 _NO_FILL_VALUE = utils.ReprObject('<no-fill-value>')
 _DEFAULT_NAME = utils.ReprObject('<default-name>')
@@ -196,7 +195,6 @@ def apply_dataarray_ufunc(func, *args, **kwargs):
     signature = kwargs.pop('signature')
     join = kwargs.pop('join', 'inner')
     exclude_dims = kwargs.pop('exclude_dims', _DEFAULT_FROZEN_SET)
-    keep_attrs = kwargs.pop('keep_attrs', False)
     if kwargs:
         raise TypeError('apply_dataarray_ufunc() got unexpected keyword '
                         'arguments: %s' % list(kwargs))
@@ -218,11 +216,6 @@ def apply_dataarray_ufunc(func, *args, **kwargs):
         coords, = result_coords
         out = DataArray(result_var, coords, name=name, fastpath=True)
 
-    if keep_attrs and isinstance(args[0], DataArray):
-        if isinstance(out, tuple):
-            out = tuple(ds._copy_attrs_from(args[0]) for ds in out)
-        else:
-            out._copy_attrs_from(args[0])
     return out
 
 
@@ -527,6 +520,7 @@ def apply_variable_ufunc(func, *args, **kwargs):
     dask = kwargs.pop('dask', 'forbidden')
     output_dtypes = kwargs.pop('output_dtypes', None)
     output_sizes = kwargs.pop('output_sizes', None)
+    keep_attrs = kwargs.pop('keep_attrs', False)
     if kwargs:
         raise TypeError('apply_variable_ufunc() got unexpected keyword '
                         'arguments: %s' % list(kwargs))
@@ -568,11 +562,17 @@ def apply_variable_ufunc(func, *args, **kwargs):
     if signature.num_outputs > 1:
         output = []
         for dims, data in zip(output_dims, result_data):
-            output.append(Variable(dims, data))
+            var = Variable(dims, data)
+            if keep_attrs and isinstance(args[0], Variable):
+                var.attrs.update(args[0].attrs)
+            output.append(var)
         return tuple(output)
     else:
         dims, = output_dims
-        return Variable(dims, result_data)
+        var = Variable(dims, result_data)
+        if keep_attrs and isinstance(args[0], Variable):
+            var.attrs.update(args[0].attrs)
+        return var
 
 
 def _apply_with_dask_atop(func, args, input_dims, output_dims, signature,
@@ -699,7 +699,7 @@ def apply_ufunc(func, *args, **kwargs):
         on each input argument that should not be broadcast. By default, we
         assume there are no core dimensions on any input arguments.
 
-        For example ,``input_core_dims=[[], ['time']]`` indicates that all
+        For example, ``input_core_dims=[[], ['time']]`` indicates that all
         dimensions on the first argument and all dimensions other than 'time'
         on the second argument should be broadcast.
 
@@ -884,14 +884,26 @@ def apply_ufunc(func, *args, **kwargs):
         func = functools.partial(func, **kwargs_)
 
     if vectorize:
-        func = np.vectorize(func,
-                            otypes=output_dtypes,
-                            signature=signature.to_gufunc_string(),
-                            excluded=set(kwargs))
+        if signature.all_core_dims:
+            # we need the signature argument
+            if LooseVersion(np.__version__) < '1.12':  # pragma: no cover
+                raise NotImplementedError(
+                    'numpy 1.12 or newer required when using vectorize=True '
+                    'in xarray.apply_ufunc with non-scalar output core '
+                    'dimensions.')
+            func = np.vectorize(func,
+                                otypes=output_dtypes,
+                                signature=signature.to_gufunc_string(),
+                                excluded=set(kwargs))
+        else:
+            func = np.vectorize(func,
+                                otypes=output_dtypes,
+                                excluded=set(kwargs))
 
     variables_ufunc = functools.partial(apply_variable_ufunc, func,
                                         signature=signature,
                                         exclude_dims=exclude_dims,
+                                        keep_attrs=keep_attrs,
                                         dask=dask,
                                         output_dtypes=output_dtypes,
                                         output_sizes=output_sizes)
@@ -920,14 +932,106 @@ def apply_ufunc(func, *args, **kwargs):
         return apply_dataarray_ufunc(variables_ufunc, *args,
                                      signature=signature,
                                      join=join,
-                                     exclude_dims=exclude_dims,
-                                     keep_attrs=keep_attrs)
+                                     exclude_dims=exclude_dims)
     elif any(isinstance(a, Variable) for a in args):
         return variables_ufunc(*args)
     else:
         return apply_array_ufunc(func, *args, dask=dask)
 
 
+def dot(*arrays, **kwargs):
+    """ dot(*arrays, dims=None)
+
+    Generalized dot product for xarray objects. Like np.einsum, but
+    provides a simpler interface based on array dimensions.
+
+    Parameters
+    ----------
+    arrays: DataArray (or Variable) objects
+        Arrays to compute.
+    dims: str or tuple of strings, optional
+        Which dimensions to sum over.
+        If not speciified, then all the common dimensions are summed over.
+    **kwargs: dict
+        Additional keyword arguments passed to numpy.einsum or
+        dask.array.einsum
+
+    Returns
+    -------
+    dot: DataArray
+
+    Examples
+    --------
+
+    >>> da_a = xr.DataArray(np.arange(3 * 4).reshape(3, 4), dims=['a', 'b'])
+    >>> da_b = xr.DataArray(np.arange(3 * 4 * 5).reshape(3, 4, 5),
+    >>>                     dims=['a', 'b', 'c'])
+    >>> da_c = xr.DataArray(np.arange(5 * 6).reshape(5, 6), dims=['c', 'd'])
+    >>>
+    >>> xr.dot(da_a, da_b, dims=['a', 'b']).dims
+    ('c', )
+    >>> xr.dot(da_a, da_b, dims=['a']).dims
+    ('b', 'c')
+    >>> xr.dot(da_a, da_b, da_c, dims=['b', 'c']).dims
+    ('a', 'd')
+    """
+    from .dataarray import DataArray
+    from .variable import Variable
+
+    dims = kwargs.pop('dims', None)
+
+    if any(not isinstance(arr, (Variable, DataArray)) for arr in arrays):
+        raise TypeError('Only xr.DataArray and xr.Variable are supported.'
+                        'Given {}.'.format([type(arr) for arr in arrays]))
+
+    if len(arrays) == 0:
+        raise TypeError('At least one array should be given.')
+
+    if isinstance(dims, basestring):
+        dims = (dims, )
+
+    common_dims = set.intersection(*[set(arr.dims) for arr in arrays])
+    all_dims = []
+    for arr in arrays:
+        all_dims += [d for d in arr.dims if d not in all_dims]
+
+    einsum_axes = 'abcdefghijklmnopqrstuvwxyz'
+    dim_map = {d: einsum_axes[i] for i, d in enumerate(all_dims)}
+
+    if dims is None:
+        # find dimensions that occur more than one times
+        dim_counts = Counter()
+        for arr in arrays:
+            dim_counts.update(arr.dims)
+        dims = tuple(d for d, c in dim_counts.items() if c > 1)
+
+    dims = tuple(dims)  # make dims a tuple
+
+    # dimensions to be parallelized
+    broadcast_dims = tuple(d for d in all_dims
+                           if d in common_dims and d not in dims)
+    input_core_dims = [[d for d in arr.dims if d not in broadcast_dims]
+                       for arr in arrays]
+    output_core_dims = [tuple(d for d in all_dims if d not in
+                              dims + broadcast_dims)]
+
+    # construct einsum subscripts, such as '...abc,...ab->...c'
+    # Note: input_core_dims are always moved to the last position
+    subscripts_list = ['...' + ''.join([dim_map[d] for d in ds]) for ds
+                       in input_core_dims]
+    subscripts = ','.join(subscripts_list)
+    subscripts += '->...' + ''.join([dim_map[d] for d in output_core_dims[0]])
+
+    # subscripts should be passed to np.einsum as arg, not as kwargs. We need
+    # to construct a partial function for apply_ufunc to work.
+    func = functools.partial(duck_array_ops.einsum, subscripts, **kwargs)
+    result = apply_ufunc(func, *arrays,
+                         input_core_dims=input_core_dims,
+                         output_core_dims=output_core_dims,
+                         dask='allowed')
+    return result.transpose(*[d for d in all_dims if d in result.dims])
+
+
 def where(cond, x, y):
     """Return elements from `x` or `y` depending on `cond`.
 
