@@ -1,31 +1,36 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
+
 import functools
 
 import numpy as np
 
 from .. import Variable
 from ..core import indexing
+from ..core.pycompat import OrderedDict, bytes_type, iteritems, unicode_type
 from ..core.utils import FrozenOrderedDict, close_on_error
-from ..core.pycompat import iteritems, bytes_type, unicode_type, OrderedDict
-
-from .common import WritableCFDataStore, DataStorePickleMixin, find_root
-from .netCDF4_ import (_nc4_group, _encode_nc4_variable, _get_datatype,
-                       _extract_nc4_variable_encoding, BaseNetCDF4Array)
+from .common import (
+    HDF5_LOCK, DataStorePickleMixin, WritableCFDataStore, find_root)
+from .netCDF4_ import (
+    BaseNetCDF4Array, _encode_nc4_variable, _extract_nc4_variable_encoding,
+    _get_datatype, _nc4_group)
 
 
 class H5NetCDFArrayWrapper(BaseNetCDF4Array):
     def __getitem__(self, key):
-        key = indexing.unwrap_explicit_indexer(
-            key, self, allow=(indexing.BasicIndexer, indexing.OuterIndexer))
+        key, np_inds = indexing.decompose_indexer(
+            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR)
+
         # h5py requires using lists for fancy indexing:
         # https://github.com/h5py/h5py/issues/992
-        # OuterIndexer only holds 1D integer ndarrays, so it's safe to convert
-        # them to lists.
-        key = tuple(list(k) if isinstance(k, np.ndarray) else k for k in key)
+        key = tuple(list(k) if isinstance(k, np.ndarray) else k for k in
+                    key.tuple)
         with self.datastore.ensure_open(autoclose=True):
-            return self.get_array()[key]
+            array = self.get_array()[key]
+
+        if len(np_inds.tuple) > 0:
+            array = indexing.NumpyIndexingAdapter(array)[np_inds]
+
+        return array
 
 
 def maybe_decode_bytes(txt):
@@ -40,21 +45,21 @@ def _read_attributes(h5netcdf_var):
     # to ensure conventions decoding works properly on Python 3, decode all
     # bytes attributes to strings
     attrs = OrderedDict()
-    for k in h5netcdf_var.ncattrs():
-        v = h5netcdf_var.getncattr(k)
+    for k, v in h5netcdf_var.attrs.items():
         if k not in ['_FillValue', 'missing_value']:
             v = maybe_decode_bytes(v)
         attrs[k] = v
     return attrs
 
 
-_extract_h5nc_encoding = functools.partial(_extract_nc4_variable_encoding,
-                                           lsd_okay=False, backend='h5netcdf')
+_extract_h5nc_encoding = functools.partial(
+    _extract_nc4_variable_encoding,
+    lsd_okay=False, h5py_okay=True, backend='h5netcdf')
 
 
 def _open_h5netcdf_group(filename, mode, group):
-    import h5netcdf.legacyapi
-    ds = h5netcdf.legacyapi.Dataset(filename, mode=mode)
+    import h5netcdf
+    ds = h5netcdf.File(filename, mode=mode)
     with close_on_error(ds):
         return _nc4_group(ds, group, mode)
 
@@ -64,12 +69,12 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
     """
 
     def __init__(self, filename, mode='r', format=None, group=None,
-                 writer=None, autoclose=False):
+                 writer=None, autoclose=False, lock=HDF5_LOCK):
         if format not in [None, 'NETCDF4']:
             raise ValueError('invalid format for h5netcdf backend')
         opener = functools.partial(_open_h5netcdf_group, filename, mode=mode,
                                    group=group)
-        self.ds = opener()
+        self._ds = opener()
         if autoclose:
             raise NotImplementedError('autoclose=True is not implemented '
                                       'for the h5netcdf backend pending '
@@ -81,20 +86,29 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
         self._opener = opener
         self._filename = filename
         self._mode = mode
-        super(H5NetCDFStore, self).__init__(writer)
+        super(H5NetCDFStore, self).__init__(writer, lock=lock)
 
     def open_store_variable(self, name, var):
         with self.ensure_open(autoclose=False):
             dimensions = var.dimensions
-            data = indexing.LazilyIndexedArray(
+            data = indexing.LazilyOuterIndexedArray(
                 H5NetCDFArrayWrapper(name, self))
             attrs = _read_attributes(var)
 
             # netCDF4 specific encoding
-            encoding = dict(var.filters())
-            chunking = var.chunking()
-            encoding['chunksizes'] = chunking \
-                if chunking != 'contiguous' else None
+            encoding = {
+                'chunksizes': var.chunks,
+                'fletcher32': var.fletcher32,
+                'shuffle': var.shuffle,
+            }
+            # Convert h5py-style compression options to NetCDF4-Python
+            # style, if possible
+            if var.compression == 'gzip':
+                encoding['zlib'] = True
+                encoding['complevel'] = var.compression_opts
+            elif var.compression is not None:
+                encoding['compression'] = var.compression
+                encoding['compression_opts'] = var.compression_opts
 
             # save source so __repr__ can detect if it's local or not
             encoding['source'] = self._filename
@@ -125,14 +139,14 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
     def set_dimension(self, name, length, is_unlimited=False):
         with self.ensure_open(autoclose=False):
             if is_unlimited:
-                self.ds.createDimension(name, size=None)
+                self.ds.dimensions[name] = None
                 self.ds.resize_dimension(name, length)
             else:
-                self.ds.createDimension(name, size=length)
+                self.ds.dimensions[name] = length
 
     def set_attribute(self, key, value):
         with self.ensure_open(autoclose=False):
-            self.ds.setncattr(key, value)
+            self.ds.attrs[key] = value
 
     def encode_variable(self, variable):
         return _encode_nc4_variable(variable)
@@ -144,8 +158,8 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
         attrs = variable.attrs.copy()
         dtype = _get_datatype(variable)
 
-        fill_value = attrs.pop('_FillValue', None)
-        if dtype is str and fill_value is not None:
+        fillvalue = attrs.pop('_FillValue', None)
+        if dtype is str and fillvalue is not None:
             raise NotImplementedError(
                 'h5netcdf does not yet support setting a fill value for '
                 'variable-length strings '
@@ -161,23 +175,49 @@ class H5NetCDFStore(WritableCFDataStore, DataStorePickleMixin):
                                           raise_on_invalid=check_encoding)
         kwargs = {}
 
-        for key in ['zlib', 'complevel', 'shuffle',
-                    'chunksizes', 'fletcher32']:
+        # Convert from NetCDF4-Python style compression settings to h5py style
+        # If both styles are used together, h5py takes precedence
+        # If set_encoding=True, raise ValueError in case of mismatch
+        if encoding.pop('zlib', False):
+            if (check_encoding and encoding.get('compression')
+                    not in (None, 'gzip')):
+                raise ValueError("'zlib' and 'compression' encodings mismatch")
+            encoding.setdefault('compression', 'gzip')
+
+        if (check_encoding and encoding.get('complevel') not in
+                (None, encoding.get('compression_opts'))):
+            raise ValueError("'complevel' and 'compression_opts' encodings "
+                             "mismatch")
+        complevel = encoding.pop('complevel', 0)
+        if complevel != 0:
+            encoding.setdefault('compression_opts', complevel)
+
+        encoding['chunks'] = encoding.pop('chunksizes', None)
+
+        for key in ['compression', 'compression_opts', 'shuffle',
+                    'chunks', 'fletcher32']:
             if key in encoding:
                 kwargs[key] = encoding[key]
-        if name not in self.ds.variables:
-            nc4_var = self.ds.createVariable(name, dtype, variable.dims,
-                                             fill_value=fill_value, **kwargs)
+        if name not in self.ds:
+            nc4_var = self.ds.create_variable(
+                name, dtype=dtype, dimensions=variable.dims,
+                fillvalue=fillvalue, **kwargs)
         else:
-            nc4_var = self.ds.variables[name]
+            nc4_var = self.ds[name]
 
         for k, v in iteritems(attrs):
-            nc4_var.setncattr(k, v)
-        return nc4_var, variable.data
+            nc4_var.attrs[k] = v
 
-    def sync(self):
+        target = H5NetCDFArrayWrapper(name, self)
+
+        return target, variable.data
+
+    def sync(self, compute=True):
+        if not compute:
+            raise NotImplementedError(
+                'compute=False is not supported for the h5netcdf backend yet')
         with self.ensure_open(autoclose=True):
-            super(H5NetCDFStore, self).sync()
+            super(H5NetCDFStore, self).sync(compute=compute)
             self.ds.sync()
 
     def close(self):
