@@ -1,6 +1,5 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
+
 import functools
 import operator
 import warnings
@@ -8,16 +7,16 @@ from distutils.version import LooseVersion
 
 import numpy as np
 
-from .. import conventions
-from .. import Variable
-from ..conventions import pop_to
+from .. import Variable, coding
+from ..coding.variables import pop_to
 from ..core import indexing
-from ..core.utils import (FrozenOrderedDict, close_on_error, is_remote_uri)
-from ..core.pycompat import iteritems, basestring, OrderedDict, PY3, suppress
-
-from .common import (WritableCFDataStore, robust_getitem, BackendArray,
-                     DataStorePickleMixin, find_root)
-from .netcdf3 import (encode_nc3_attr_value, encode_nc3_variable)
+from ..core.pycompat import (
+    PY3, OrderedDict, basestring, iteritems, suppress)
+from ..core.utils import FrozenOrderedDict, close_on_error, is_remote_uri
+from .common import (
+    HDF5_LOCK, BackendArray, DataStorePickleMixin, WritableCFDataStore,
+    find_root, robust_getitem)
+from .netcdf3 import encode_nc3_attr_value, encode_nc3_variable
 
 # This lookup table maps from dtype.byteorder to a readable endian
 # string used by netCDF4.
@@ -43,6 +42,11 @@ class BaseNetCDF4Array(BackendArray):
             dtype = np.dtype('O')
         self.dtype = dtype
 
+    def __setitem__(self, key, value):
+        with self.datastore.ensure_open(autoclose=True):
+            data = self.get_array()
+            data[key] = value
+
     def get_array(self):
         self.datastore.assert_open()
         return self.datastore.ds.variables[self.variable_name]
@@ -50,9 +54,8 @@ class BaseNetCDF4Array(BackendArray):
 
 class NetCDF4ArrayWrapper(BaseNetCDF4Array):
     def __getitem__(self, key):
-        key = indexing.unwrap_explicit_indexer(
-            key, self, allow=(indexing.BasicIndexer, indexing.OuterIndexer))
-
+        key, np_inds = indexing.decompose_indexer(
+            key, self.shape, indexing.IndexingSupport.OUTER)
         if self.datastore.is_remote:  # pragma: no cover
             getitem = functools.partial(robust_getitem, catch=RuntimeError)
         else:
@@ -60,7 +63,7 @@ class NetCDF4ArrayWrapper(BaseNetCDF4Array):
 
         with self.datastore.ensure_open(autoclose=True):
             try:
-                data = getitem(self.get_array(), key)
+                array = getitem(self.get_array(), key.tuple)
             except IndexError:
                 # Catch IndexError in netCDF4 and return a more informative
                 # error message.  This is most often called when an unsorted
@@ -73,12 +76,16 @@ class NetCDF4ArrayWrapper(BaseNetCDF4Array):
                     msg += '\n\nOriginal traceback:\n' + traceback.format_exc()
                 raise IndexError(msg)
 
-        return data
+        if len(np_inds.tuple) > 0:
+            array = indexing.NumpyIndexingAdapter(array)[np_inds]
+
+        return array
 
 
 def _encode_nc4_variable(var):
-    if var.dtype.kind == 'S':
-        var = conventions.maybe_encode_as_char_array(var)
+    for coder in [coding.strings.EncodedStringCoder(allows_unicode=True),
+                  coding.strings.CharacterArrayCoder()]:
+        var = coder.encode(var)
     return var
 
 
@@ -91,12 +98,13 @@ def _get_datatype(var, nc_format='NETCDF4'):
 
 
 def _nc4_dtype(var):
-    if var.dtype.kind == 'U':
+    if coding.strings.is_unicode_dtype(var.dtype):
         dtype = str
     elif var.dtype.kind in ['i', 'u', 'f', 'c', 'S']:
         dtype = var.dtype
     else:
-        raise ValueError('cannot infer dtype for netCDF4 variable')
+        raise ValueError('unsupported dtype for netCDF4 variable: {}'
+                         .format(var.dtype))
     return dtype
 
 
@@ -151,8 +159,8 @@ def _force_native_endianness(var):
 
 
 def _extract_nc4_variable_encoding(variable, raise_on_invalid=False,
-                                   lsd_okay=True, backend='netCDF4',
-                                   unlimited_dims=None):
+                                   lsd_okay=True, h5py_okay=False,
+                                   backend='netCDF4', unlimited_dims=None):
     if unlimited_dims is None:
         unlimited_dims = ()
 
@@ -163,6 +171,9 @@ def _extract_nc4_variable_encoding(variable, raise_on_invalid=False,
                            'chunksizes', 'shuffle', '_FillValue'])
     if lsd_okay:
         valid_encodings.add('least_significant_digit')
+    if h5py_okay:
+        valid_encodings.add('compression')
+        valid_encodings.add('compression_opts')
 
     if not raise_on_invalid and encoding.get('chunksizes') is not None:
         # It's possible to get encoded chunksizes larger than a dimension size
@@ -224,6 +235,31 @@ def _disable_auto_decode_group(ds):
         _disable_auto_decode_variable(var)
 
 
+def _is_list_of_strings(value):
+    if (np.asarray(value).dtype.kind in ['U', 'S'] and
+            np.asarray(value).size > 1):
+        return True
+    else:
+        return False
+
+
+def _set_nc_attribute(obj, key, value):
+    if _is_list_of_strings(value):
+        # encode as NC_STRING if attr is list of strings
+        try:
+            obj.setncattr_string(key, value)
+        except AttributeError:
+            # Inform users with old netCDF that does not support
+            # NC_STRING that we can't serialize lists of strings
+            # as attrs
+            msg = ('Attributes which are lists of strings are not '
+                   'supported with this version of netCDF. Please '
+                   'upgrade to netCDF4-python 1.2.4 or greater.')
+            raise AttributeError(msg)
+    else:
+        obj.setncattr(key, value)
+
+
 class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
     """Store for reading and writing data via the Python-NetCDF4 library.
 
@@ -231,14 +267,14 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
     """
 
     def __init__(self, netcdf4_dataset, mode='r', writer=None, opener=None,
-                 autoclose=False):
+                 autoclose=False, lock=HDF5_LOCK):
 
         if autoclose and opener is None:
             raise ValueError('autoclose requires an opener')
 
         _disable_auto_decode_group(netcdf4_dataset)
 
-        self.ds = netcdf4_dataset
+        self._ds = netcdf4_dataset
         self._autoclose = autoclose
         self._isopen = True
         self.format = self.ds.data_model
@@ -249,23 +285,23 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
             self._opener = functools.partial(opener, mode=self._mode)
         else:
             self._opener = opener
-        super(NetCDF4DataStore, self).__init__(writer)
+        super(NetCDF4DataStore, self).__init__(writer, lock=lock)
 
     @classmethod
     def open(cls, filename, mode='r', format='NETCDF4', group=None,
              writer=None, clobber=True, diskless=False, persist=False,
-             autoclose=False):
+             autoclose=False, lock=HDF5_LOCK):
         import netCDF4 as nc4
         if (len(filename) == 88 and
                 LooseVersion(nc4.__version__) < "1.3.1"):
             warnings.warn(
-                '\nA segmentation fault may occur when the\n'
-                'file path has exactly 88 characters as it does.\n'
-                'in this case. The issue is known to occur with\n'
-                'version 1.2.4 of netCDF4 and can be addressed by\n'
-                'upgrading netCDF4 to at least version 1.3.1.\n'
-                'More details can be found here:\n'
-                'https://github.com/pydata/xarray/issues/1745  \n')
+                'A segmentation fault may occur when the '
+                'file path has exactly 88 characters as it does '
+                'in this case. The issue is known to occur with '
+                'version 1.2.4 of netCDF4 and can be addressed by '
+                'upgrading netCDF4 to at least version 1.3.1. '
+                'More details can be found here: '
+                'https://github.com/pydata/xarray/issues/1745')
         if format is None:
             format = 'NETCDF4'
         opener = functools.partial(_open_netcdf4_group, filename, mode=mode,
@@ -274,12 +310,13 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
                                    format=format)
         ds = opener()
         return cls(ds, mode=mode, writer=writer, opener=opener,
-                   autoclose=autoclose)
+                   autoclose=autoclose, lock=lock)
 
     def open_store_variable(self, name, var):
         with self.ensure_open(autoclose=False):
             dimensions = var.dimensions
-            data = indexing.LazilyIndexedArray(NetCDF4ArrayWrapper(name, self))
+            data = indexing.LazilyOuterIndexedArray(
+                NetCDF4ArrayWrapper(name, self))
             attributes = OrderedDict((k, var.getncattr(k))
                                      for k in var.ncattrs())
             _ensure_fill_value_valid(data, attributes)
@@ -341,7 +378,7 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
         with self.ensure_open(autoclose=False):
             if self.format != 'NETCDF4':
                 value = encode_nc3_attr_value(value)
-            self.ds.setncattr(key, value)
+            _set_nc_attribute(self.ds, key, value)
 
     def set_variables(self, *args, **kwargs):
         with self.ensure_open(autoclose=False):
@@ -396,13 +433,15 @@ class NetCDF4DataStore(WritableCFDataStore, DataStorePickleMixin):
         for k, v in iteritems(attrs):
             # set attributes one-by-one since netCDF4<1.0.10 can't handle
             # OrderedDict as the input to setncatts
-            nc4_var.setncattr(k, v)
+            _set_nc_attribute(nc4_var, k, v)
 
-        return nc4_var, variable.data
+        target = NetCDF4ArrayWrapper(name, self)
 
-    def sync(self):
+        return target, variable.data
+
+    def sync(self, compute=True):
         with self.ensure_open(autoclose=True):
-            super(NetCDF4DataStore, self).sync()
+            super(NetCDF4DataStore, self).sync(compute=compute)
             self.ds.sync()
 
     def close(self):
