@@ -758,10 +758,23 @@ class DataArray(AbstractArray, DataWithCoords):
         """Return a new DataArray whose dataset is given by selecting
         index labels along the specified dimension(s).
 
+        .. warning::
+
+          Do not try to assign values when using any of the indexing methods
+          ``isel`` or ``sel``::
+
+            da = xr.DataArray([0, 1, 2, 3], dims=['x'])
+            # DO NOT do this
+            da.isel(x=[0, 1, 2])[1] = -1
+
+          Assigning values with the chained indexing using ``.sel`` or
+          ``.isel`` fails silently.
+
         See Also
         --------
         Dataset.sel
         DataArray.isel
+
         """
         ds = self._to_temp_dataset().sel(drop=drop, method=method,
                                          tolerance=tolerance, **indexers)
@@ -1443,8 +1456,7 @@ class DataArray(AbstractArray, DataWithCoords):
         return np.ma.MaskedArray(data=self.values, mask=isnull, copy=copy)
 
     def to_netcdf(self, *args, **kwargs):
-        """
-        Write DataArray contents to a netCDF file.
+        """Write DataArray contents to a netCDF file.
 
         Parameters
         ----------
