@@ -1,6 +1,5 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
+
 import pandas as pd
 
 from .alignment import deep_align
@@ -8,7 +7,6 @@ from .pycompat import OrderedDict, basestring
 from .utils import Frozen
 from .variable import as_variable, assert_unique_multiindex_level_names
 
-
 PANDAS_TYPES = (pd.Series, pd.DataFrame, pd.Panel)
 
 _VALID_COMPAT = Frozen({'identical': 0,
@@ -549,6 +547,21 @@ def dataset_merge_method(dataset, other, overwrite_vars, compat, join):
 
 
 def dataset_update_method(dataset, other):
-    """Guts of the Dataset.update method"""
+    """Guts of the Dataset.update method
+
+    This drops a duplicated coordinates from `other` (GH:2068)
+    """
+    from .dataset import Dataset
+    from .dataarray import DataArray
+
+    other = other.copy()
+    for k, obj in other.items():
+        if isinstance(obj, (Dataset, DataArray)):
+            # drop duplicated coordinates
+            coord_names = [c for c in obj.coords
+                           if c not in obj.dims and c in dataset.coords]
+            if coord_names:
+                other[k] = obj.drop(coord_names)
+
     return merge_core([dataset, other], priority_arg=1,
                       indexes=dataset.indexes)
